import 'dart:async';
import 'dart:io';

import 'package:flulm_auth/src/page/auth_page.dart';
import 'package:flulm_auth/src/page/pay_page.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import '../flulm_auth.dart';

StreamController<LMAuthResponse> _responseAuthController = new StreamController.broadcast();

StreamController<LMAuthResponse> _responsePayController = new StreamController.broadcast();

/// Response from auth  自动登录
Stream<LMAuthResponse> get responseFromAuth => _responseAuthController.stream;

Stream<LMAuthResponse> get responseFromPay => _responsePayController.stream;

MethodChannel _channel = const MethodChannel('com.awesome/flulm_auth')..setMethodCallHandler(_handler);

Future<dynamic> _handler(MethodCall methodCall) {
  if ("onAuthResponse" == methodCall.method) {
    _responseAuthController.add(
      LMAuthResponse.fromMap(methodCall.arguments),
    );
  } else if ("onPayResponse" == methodCall.method) {
    _responsePayController.add(
      LMAuthResponse.fromMap(methodCall.arguments),
    );
  }
  return Future.value(true);
}

Future<void> sendAuth({
  @required BuildContext mContext,
  @required String openId,
  @required String scope,
  String language = "en",
  bool supportScan = false,
  int isTest = 0,
}) async {
  assert(scope != null && scope.trim().isNotEmpty);
  bool isInstalledLema = false;
  if (Platform.isAndroid) {
    isInstalledLema = await isLemaPayInstalled();
  }
  print("install->$isInstalledLema");

  if (!isInstalledLema) {
    final result = await Navigator.push(
      mContext,
      CupertinoPageRoute(
        builder: (context) => AuthPage(
          scope: scope,
          appid: openId,
          isTest: isTest,
          language: language,
          supportScan: supportScan,
        ),
      ),
    );
    if (null != result) {
      print("返回的内容=$result");

      _responseAuthController.add(
        LMAuthResponse.fromMap(result),
      );
    }
  } else {
    await _channel.invokeMethod("sendAuth", {"scope": scope});
  }
}

Future<bool> isLemaPayInstalled() => _channel.invokeMethod("isLemaPayInstalled").then((install) {
      return install;
    });

Future<int> isLemaPaySupportAPI() => _channel.invokeMethod("isLemaPaySupportAPI").then((version) {
      return version;
    });

///调用乐马支付
Future<void> sendPay({
  @required BuildContext mContext,
  @required String openId,
  @required String signData,
  String language = "en",
  bool supportScan = false,
  int isTest = 0,
}) async {
  _sendCore(mContext: mContext, language:language, openId: openId, signData: signData, isTest: isTest, payType: "order");
}

///调用乐马提现
Future<void> sendCredit({
  @required BuildContext mContext,
  @required String openId,
  @required String signData,
  String language = "en",
  int isTest = 0,
}) async {
  _sendCore(mContext: mContext, openId: openId, signData: signData, payType: "credit_getout");
}

///调用乐马预授权
Future<void> sendFrozen({
  @required BuildContext mContext,
  @required String openId,
  @required String signData,
  String language = "en",
  int isTest = 0,
}) async {
  _sendCore(mContext: mContext, openId: openId, signData: signData, payType: "frozen");
}

//调用乐马签约
//Future<void> sendSign({
//  @required BuildContext mContext,
//  @required String openId,
//  @required String signData,
//  String language = "en",
//  int isTest = 0,
//}) async {
//  _sendCore(
//      mContext: mContext, openId: openId, signData: signData, payType: "autopay_sign");
//}

///调用乐马支付相关SDK
Future<void> _sendCore({
  @required BuildContext mContext,
  @required String openId,
  @required String signData,
  @required String payType,
  bool supportScan = false,
  String language = "en",
  int isTest = 0,
}) async {
  assert(
      mContext != null && openId != null && openId.trim().isNotEmpty && signData != null && signData.trim().isNotEmpty);
  bool isInstalledLema = await isLemaPayInstalled();

  if (!isInstalledLema) {
    final result = await Navigator.push(
      mContext,
      CupertinoPageRoute(
        builder: (context) => PayPage(
          type: payType,
          signData: signData,
          isTest: isTest,
          language: language,
          supportScan: supportScan,
        ),
//            AuthPage(),
      ),
    );
    if (null != result) {
      print("返回的内容=$result");

      _responsePayController.add(
        LMAuthResponse.fromMap(result),
      );
    }
  } else {
    await _channel.invokeMethod("sendPay", {
      "openId": openId,
      "sign_data": signData,
      "language": language,
      "pay_type": payType,
      "isTest": isTest,
    });
  }
}

///[appId] is not necessary.
///if [doOnIOS] is true ,fluwx will register WXApi on iOS.
///if [doOnAndroid] is true, fluwx will register WXApi on Android.
Future register({@required String appId, bool doOnIOS: true, doOnAndroid: true, enableMTA: false}) async {
  return await _channel
      .invokeMethod("registerApp", {"appId": appId, "iOS": doOnIOS, "android": doOnAndroid, "enableMTA": enableMTA});
}
